/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_termstats();
extern void test_stubdb1();
extern void test_stubdb2();
extern void test_stubdb3();
extern void test_stubdb4();
extern void test_stubdb5();
extern void test_stubdb6();
extern void test_stubdb8();
extern void test_matchdecider1();
extern void test_matchdecider2();
extern void test_matchdecider3();
extern void test_msetiterator1();
extern void test_msetiterator2();
extern void test_msetiterator3();
extern void test_esetiterator1();
extern void test_esetiterator2();
extern void test_collapsekey1();
extern void test_collapsekey2();
extern void test_collapsekey3();
extern void test_collapsekey4();
extern void test_keepalive1();
extern void test_allterms1();
extern void test_allterms2();
extern void test_allterms3();
extern void test_allterms4();
extern void test_allterms5();
extern void test_allterms6();
extern void test_specialterms1();
extern void test_specialterms2();
extern void test_rsetmultidb2();
extern void test_multiexpand1();
extern void test_postlist1();
extern void test_postlist2();
extern void test_postlist3();
extern void test_postlist4();
extern void test_postlist5();
extern void test_postlist6();
extern void test_collfreq1();
extern void test_sortvalue1();
extern void test_consistency1();
extern void test_chertdatabasenotfounderror1();
extern void test_glassdatabasenotfounderror1();
extern void test_databasenotfounderror2();
extern void test_chertdatabaseopen1();
extern void test_sortrel1();
extern void test_netstats1();
extern void test_userweight1();
extern void test_matchall1();
extern void test_valuesetmatchdecider2();
