/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_compactnorenumber1();
extern void test_compactmerge1();
extern void test_compactmultichunks1();
extern void test_compactstub1();
extern void test_compactstub2();
extern void test_compactstub3();
extern void test_compactstub4();
extern void test_compactmissingtables1();
extern void test_compactmergesynonym1();
extern void test_compactempty1();
extern void test_compactmultipass1();
extern void test_compacttofd1();
extern void test_compacttofd2();
extern void test_compactsingle1();
extern void test_compact1();
