/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_synonym1();
extern void test_synonym2();
extern void test_synonym3();
extern void test_synonym4();
extern void test_opmax1();
