/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_valuestream1();
extern void test_valuestream2();
extern void test_valuestream3();
extern void test_valueweightsource5();
extern void test_valuemapsource1();
extern void test_valuemapsource2();
extern void test_fixedweightsource2();
extern void test_decvalwtsource1();
extern void test_decvalwtsource2();
extern void test_decvalwtsource3();
extern void test_decvalwtsource4();
extern void test_decvalwtsource5();
