/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_queryterms1();
extern void test_matchall2();
extern void test_matchnothing1();
extern void test_overload1();
extern void test_possubqueries1();
extern void test_xor3();
extern void test_nonutf8termdesc1();
extern void test_queryintro1();
extern void test_phrasealldocs1();
extern void test_wildcard1();
extern void test_wildcard2();
extern void test_wildcard4();
extern void test_dualprefixwildcard1();
extern void test_loosephrase1();
extern void test_loosenear1();
extern void test_complexphrase1();
extern void test_complexnear1();
extern void test_complexphrase2();
extern void test_complexnear2();
extern void test_zeroestimate1();
extern void test_complexphrase3();
extern void test_complexnear3();
extern void test_subdbwithoutpos1();
extern void test_notandor1();
extern void test_boolorbug1();
extern void test_hoistnotbug1();
extern void test_emptynot1();
extern void test_emptymaybe1();
extern void test_phraseweightcheckbug1();
extern void test_orphanedhint1();
