/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_spell0();
extern void test_spell1();
extern void test_spell2();
extern void test_spell3();
extern void test_spell4();
extern void test_spell5();
extern void test_spell6();
extern void test_spell7();
extern void test_spell8();
extern void test_spell10();
