/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_externalsource1();
extern void test_externalsource2();
extern void test_externalsource3();
extern void test_externalsource4();
extern void test_valueweightsource1();
extern void test_valueweightsource2();
extern void test_valueweightsource3();
extern void test_fixedweightsource1();
extern void test_changemaxweightsource1();
extern void test_emptyvalwtsource1();
extern void test_matchtimelimit1();
extern void test_postingsourcebounds1();
extern void test_postingsourceclone1();
extern void test_postingsourceshardindex1();
extern void test_estimaterounding1();
