/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_sortfunctor1();
extern void test_sortfunctor2();
extern void test_sortfunctor3();
extern void test_changesorter1();
extern void test_sortfunctorempty1();
extern void test_multivaluekeymaker1();
extern void test_sortfunctorremote1();
