/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_valuerange1();
extern void test_valuerange2();
extern void test_valuerange5();
extern void test_valuerange6();
extern void test_valuerange7();
extern void test_valuege1();
extern void test_valuege2();
extern void test_valuele1();
extern void test_valuege3();
extern void test_valuege4();
extern void test_valuerange3();
extern void test_valuerange4();
extern void test_valuerangematchesub1();
