/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_lockfileumask1();
extern void test_totaldoclen1();
extern void test_exceed32bitcombineddb1();
extern void test_dbstats1();
extern void test_dbstats2();
extern void test_alldocspl3();
extern void test_modifiedpostlist1();
extern void test_doclenaftercommit1();
extern void test_valuesaftercommit1();
extern void test_lockfilefd0or1();
extern void test_lockfilealreadyopen1();
extern void test_testlock1();
extern void test_testlock2();
extern void test_testlock3();
extern void test_testlock4();
extern void test_matchdecider4();
extern void test_replacedoc7();
extern void test_replacedoc8();
extern void test_replacedoc9();
extern void test_databasemodified1();
extern void test_qpmemoryleak1();
extern void test_msize1();
extern void test_msize2();
extern void test_xordecay1();
extern void test_ordecay1();
extern void test_ordecay2();
extern void test_orcheck1();
extern void test_failedreplace1();
extern void test_failedreplace2();
extern void test_phrase3();
extern void test_msetfirst2();
extern void test_emptydb1();
extern void test_multiargop1();
extern void test_stubdb7();
extern void test_msetweights1();
extern void test_itorskiptofromend1();
extern void test_blocksize1();
extern void test_notermlist1();
extern void test_newfreelistblock1();
extern void test_readonlyparentdir1();
extern void test_phrasebug1();
extern void test_retrylock1();
extern void test_dbfilefd012();
extern void test_cursorbug1();
extern void test_sortvalue2();
extern void test_enquiregetquery1();
extern void test_embedded1();
extern void test_exactxor1();
extern void test_getrevision1();
extern void test_getrevision2();
extern void test_getdocumentlazy1();
extern void test_getdocumentlazy2();
extern void test_getuniqueterms1();
extern void test_nopositionbug1();
extern void test_checkatleast4();
extern void test_freelistleak1();
extern void test_splitpostings1();
extern void test_multidb1();
extern void test_allterms7();
extern void test_nosuchterm();
extern void test_unsupportedcheck1();
extern void test_unsupportedcheck2();
extern void test_unsupportedcheck3();
extern void test_corruptglass1();
