/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_zerodocid1();
extern void test_emptyquery1();
extern void test_simplequery1();
extern void test_simplequery2();
extern void test_simplequery3();
extern void test_multidb2();
extern void test_msetmaxitems1();
extern void test_expandweights1();
extern void test_expandweights2();
extern void test_expandweights3();
extern void test_expandweights4();
extern void test_expandweights5();
extern void test_expandweights6();
extern void test_expandweights7();
extern void test_expandweights8();
extern void test_expandmaxitems1();
extern void test_boolquery1();
extern void test_msetfirst1();
extern void test_topercent1();
extern void test_topercent2();
extern void test_expandfunctor1();
extern void test_expanddeciderfilterprefix2();
extern void test_pctcutoff1();
extern void test_pctcutoff2();
extern void test_pctcutoff3();
extern void test_cutoff1();
extern void test_allowqterms1();
extern void test_maxattain1();
extern void test_reversebool1();
extern void test_reversebool2();
extern void test_getmterms1();
extern void test_getmterms2();
extern void test_repeatquery1();
extern void test_fetchdocs1();
extern void test_absentterm1();
extern void test_absentterm2();
extern void test_rset1();
extern void test_rset2();
extern void test_rsetmultidb1();
extern void test_rsetmultidb3();
extern void test_eliteset1();
extern void test_elitesetmulti1();
extern void test_eliteset2();
extern void test_elitesetmulti2();
extern void test_eliteset3();
extern void test_eliteset4();
extern void test_elitesetmulti4();
extern void test_eliteset5();
extern void test_termlisttermfreq1();
extern void test_qterminfo1();
extern void test_qterminfo2();
extern void test_msetzeroitems1();
extern void test_matches1();
extern void test_wqf1();
extern void test_qlen1();
extern void test_termlist1();
extern void test_termlist2();
extern void test_termlist3();
extern void test_termlist4();
extern void test_puncterms1();
extern void test_spaceterms1();
extern void test_xor1();
extern void test_xor2();
extern void test_getdoc1();
extern void test_emptyop1();
extern void test_checkatleast1();
extern void test_checkatleast2();
extern void test_checkatleast3();
extern void test_allpostlist1();
extern void test_emptyterm1();
extern void test_alldocspl1();
extern void test_alldocspl2();
extern void test_scaleweight1();
extern void test_scaleweight2();
extern void test_uuid1();
