/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_version1();
extern void test_nosubdatabases1();
extern void test_document1();
extern void test_document2();
extern void test_documentclearterms1();
extern void test_documentclearvalues1();
extern void test_documentemptyterm1();
extern void test_emptyquery4();
extern void test_singlesubquery1();
extern void test_singlesubquery2();
extern void test_singlesubquery3();
extern void test_pairwisequery1();
extern void test_combinewqfnomore1();
extern void test_subclassablerefcount1();
extern void test_subclassablerefcount2();
extern void test_subclassablerefcount3();
extern void test_subclassablerefcount4();
extern void test_subclassablerefcount5();
extern void test_subclassablerefcount6();
extern void test_subclassablerefcount7();
extern void test_subclassablerefcount8();
extern void test_nonutf8docdesc1();
extern void test_deletewhileiterating1();
extern void test_removepostings();
extern void test_errorcopyctor();
extern void test_stlesetiterator();
extern void test_stlmsetiterator();
extern void test_stlpositioniterator();
extern void test_stlpostingiterator();
extern void test_stltermiterator();
extern void test_stlutf8iterator();
extern void test_stlvalueiterator();
