/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_matchspy1();
extern void test_matchspy2();
extern void test_matchspy4();
extern void test_matchspy5();
extern void test_matchspy6();
extern void test_matchspy7();
