/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_valuestats1();
extern void test_valuestats2();
extern void test_valuestats3();
extern void test_valuestats4();
extern void test_valuestats5();
