/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_utf8iterator1();
extern void test_utf8iterator2();
extern void test_unicode1();
extern void test_caseconvert1();
extern void test_caseconvert2();
extern void test_utf8convert1();
extern void test_unicodepredicates1();
