/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_replicate1();
extern void test_replicate2();
extern void test_replicate3();
extern void test_replicate4();
extern void test_replicate5();
extern void test_replicate6();
extern void test_replicate7();
