/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_near1();
extern void test_near2();
extern void test_phrase1();
extern void test_phrase2();
extern void test_poslist1();
extern void test_poslist2();
extern void test_poslist3();
extern void test_positfromtermit1();
