/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_badtransaction1();
extern void test_committransaction1();
extern void test_canceltransaction1();
extern void test_canceltransaction2();
extern void test_canceltransaction3();
