/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_closedb1();
extern void test_closedb2();
extern void test_closedb3();
extern void test_closedb4();
extern void test_closedb5();
extern void test_closedb6();
extern void test_closedb7();
extern void test_closedb8();
extern void test_closedb9();
extern void test_closedb10();
