/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_compactoldnorenumber1();
extern void test_compactoldmerge1();
extern void test_compactoldmultichunks1();
extern void test_compactoldstub1();
extern void test_compactoldstub2();
extern void test_compactoldstub3();
extern void test_compactoldstub4();
extern void test_compactoldmissingtables1();
extern void test_compactoldmergesynonym1();
extern void test_compactoldempty1();
extern void test_compactoldmultipass1();
