/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_getqterms1();
extern void test_getqterms2();
extern void test_emptyquery2();
extern void test_emptyquery3();
extern void test_querylen1();
extern void test_querylen2();
extern void test_dontflattensubqueries1();
extern void test_emptyquerypart1();
extern void test_stemlangs1();
extern void test_nosuchdb1();
extern void test_addvalue1();
extern void test_poscollapse2();
extern void test_uninitdb1();
extern void test_scaleweight3();
extern void test_rset3();
extern void test_rset4();
extern void test_valuesetmatchdecider1();
extern void test_emptymset1();
extern void test_expanddeciderfilterprefix1();
