/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_stem1();
extern void test_stem2();
extern void test_stem3();
extern void test_stem4();
extern void test_stemempty1();
extern void test_stemlangs2();
