/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_defaultctor1();
extern void test_copyassign1();
extern void test_itortraits1();
extern void test_containertraits1();
