/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_consistency3();
extern void test_pctcutoff4();
extern void test_pctcutoff5();
extern void test_topercent3();
extern void test_topercent4();
extern void test_topercent5();
extern void test_topercent6();
extern void test_topercent7();
extern void test_checkzeromaxpartopt1();
