/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_weightserialisation1();
extern void test_weight1();
extern void test_bm25weight1();
extern void test_bm25weight4();
extern void test_bm25weight5();
extern void test_bm25plusweight2();
extern void test_bm25plusweight3();
extern void test_inl2weight2();
extern void test_inl2weight3();
extern void test_ifb2weight2();
extern void test_ifb2weight3();
extern void test_ineb2weight2();
extern void test_ineb2weight3();
extern void test_bb2weight2();
extern void test_bb2weight3();
extern void test_bb2weight4();
extern void test_dlhweight1();
extern void test_dlhweight3();
extern void test_pl2weight2();
extern void test_pl2weight3();
extern void test_pl2plusweight2();
extern void test_pl2plusweight4();
extern void test_pl2plusweight5();
extern void test_dphweight1();
extern void test_dphweight3();
extern void test_tfidfweight1();
extern void test_tfidfweight3();
extern void test_checkinitweight1();
extern void test_checkstatsweight1();
extern void test_checkstatsweight2();
extern void test_checkstatsweight3();
extern void test_checkstatsweight4();
extern void test_unigramlmweight4();
extern void test_unigramlmweight5();
extern void test_unigramlmweight7();
extern void test_unigramlmweight8();
extern void test_coordweight1();
