/* Warning: This file is generated by ./collate-test - do not modify directly! */
extern void test_adddoc1();
extern void test_adddoc2();
extern void test_adddoc3();
extern void test_adddoc4();
extern void test_adddoc5();
extern void test_adddoc6();
extern void test_implicitendsession1();
extern void test_databaseassign1();
extern void test_deldoc1();
extern void test_deldoc2();
extern void test_deldoc3();
extern void test_deldoc4();
extern void test_deldoc5();
extern void test_deldoc6();
extern void test_replacedoc1();
extern void test_replacedoc2();
extern void test_replacedoc3();
extern void test_replacedoc4();
extern void test_replacedoc5();
extern void test_replacedoc6();
extern void test_uniqueterm1();
extern void test_allpostlist2();
extern void test_emptyterm2();
extern void test_phraseorneartoand1();
extern void test_longpositionlist1();
extern void test_consistency2();
extern void test_crashrecovery1();
extern void test_nomoredocids1();
extern void test_synonymitor1();
extern void test_termtoolong1();
extern void test_postlist7();
extern void test_lazytablebug1();
extern void test_cursordelbug1();
extern void test_modifyvalues1();
extern void test_protocolbug1();
extern void test_remotefdleak1();
extern void test_corruptglass2();
